
const isolateBackend = require('./isolate')
const { spawn } = require('child_process');
const libSupport = require('./lib')
const fs = require('fs')


function runIsolate(filename) {
    let timeStart = Date.now()
    let {isolate, context} = isolateBackend.createIsolate();
    // Let's see what happens when we try to blow the isolate's memory
    fs.readFile(filename, 'utf-8',(err, data) => {
        
        isolate.compileScript(data).then(script => {
        script.run(context)
        .then(result => {
            console.log(result);
            let timeDifference = Math.ceil((Date.now() - timeStart))
            console.log("isolate time taken: ",  timeDifference);
        });
        })
    });
}

function runProcess(filename) {
    let timeStart = Date.now()
    const process = spawn('node', [filename]);

    process.stdout.on('data', (data) => {
        console.log(`stdout: ${data}`);
        let timeDifference = Math.ceil((Date.now() - timeStart))
        console.log("process time taken: ",  timeDifference);
    });

    process.stderr.on('data', (data) => {
        console.error(`stderr: ${data}`);
    });

    process.on('close', (code) => {
        console.log(`child process exited with code ${code}`);
    });
    
} 

function runContainer(path) {
    let buildStart = Date.now()
    let imageName = libSupport.makeid(10)
    const process = spawn('docker', ["build", "-t", imageName, path, "-q"]);

    process.stdout.on('data', (data) => {
        console.log(`stdout: ${data}`);
        
    });

    process.stderr.on('data', (data) => {
        console.error(`stderr: ${data}`);
    });

    process.on('close', (code) => {
        console.log(`child process exited with code ${code}`);
        let timeDifference = Math.ceil((Date.now() - buildStart))
        console.log("image build time taken: ",  timeDifference);
        let timeStart = Date.now()
        const process = spawn('docker', ["run", "--name", imageName, imageName]);

        process.stdout.on('data', (data) => {
            console.log(`stdout: ${data}`);
            let timeDifference = Math.ceil((Date.now() - timeStart))
            console.log("container run time taken: ",  timeDifference);
        });
    
        process.stderr.on('data', (data) => {
            console.error(`stderr: ${data}`);
        });
    
        process.on('close', (code) => {
        })
    });
}

runIsolate('./test/script.js');
runProcess('./test/script.js');
runContainer("test/");